@@ -6,9 +6,9 @@
 ;; Author: Johnson Denen <johnson.denen@gmail.com>
 ;;         Marty Hiatt <mousebot@disroot.org>
 ;; Maintainer: Marty Hiatt <mousebot@disroot.org>
-;; Package-Version: 20251201.1553
-;; Package-Revision: 3c00418bfbb1
-;; Package-Requires: ((emacs "28.1") (persist "0.8") (tp "0.7"))
+;; Package-Version: 20260301.916
+;; Package-Revision: edea11cc69c7
+;; Package-Requires: ((emacs "29.1") (persist "0.8") (tp "0.8"))
 ;; Homepage: https://codeberg.org/martianh/mastodon.el
 
 ;; This file is not part of GNU Emacs.
@@ -41,6 +41,7 @@
 (eval-when-compile (require 'subr-x))
 (require 'url)
 (require 'thingatpt)
+(require 'alert)
 (require 'shr)
 
 (require 'mastodon-http)
@@ -76,6 +77,7 @@
 (autoload 'mastodon-toot--view-toot-history "mastodon-tl")
 (autoload 'mastodon-tl-return "mastodon-tl")
 (autoload 'mastodon-tl-jump-to-followed-tag "mastodon-tl")
+(autoload 'mastodon-notifications--update-with-timer "mastodon-notifications")
 
 ;; for M-x visibility
 ;; (views.el uses `mastodon-mode-map', so we can't easily require it)
@@ -105,6 +107,8 @@
   nil :interactive)
 
 (autoload 'special-mode "simple")
+(autoload 'mastodon-tl--thread-do "mastodon-tl")
+(autoload 'mastodon-notifications--get-unread-count "mastodon-notifications")
 
 (defvar mastodon-tl--highlight-current-toot)
 (defvar mastodon-notifications--map)
@@ -115,6 +119,9 @@
   "List of notification types for which grouping is implemented.
 Used in `mastodon-notifications-get'")
 
+(defvar mastodon-notifications-notify-shown nil
+  "Whether recent notifications have been seen by the user or not.")
+
 (defgroup mastodon nil
   "Interface with Mastodon."
   :prefix "mastodon-"
@@ -194,6 +201,33 @@ A count of 2 for example means to display like so: \"Bob, Jenny
 and X others...\"."
   :type '(integer))
 
+(defcustom mastodon-notifications-alerts-interval 60
+  "How often to check for new notifications, in seconds."
+  :type '(integer))
+
+(defcustom mastodon-notifications-alerts t
+  "Whether to enable alert.el unread-notification alerts.
+If this is enabled and alert.el is installed, an alert.el notification
+will be shown using `mastodon-notifications-alert-style' as its alert
+style. If this is enabled but alert.el is not installed, alerts will be
+shown using `message'.
+Alerts are only checked for when at least 1 mastodon.el buffer is open."
+  :type '(boolean))
+
+(defcustom mastodon-notifications-alert-style alert-default-style
+  "The type of alert.el style to use for mastodon.el notification alerts.
+Currently, if you customize this variable, you need to restart Emacs for
+it to take effect, or if you don't have any other alert.el rules set up,
+you can nil `alert-internal-configuration' and reload mastodon.el"
+  :type
+  `(choice
+    ,@(append
+       '((const :tag "alert.el default"
+                :value alert-default-style))
+       (cl-loop for x in alert-styles
+                collect (list 'const :tag (plist-get (cdr x) :title)
+                              :value (car x))))))
+
 (defun mastodon-kill-window ()
   "Quit window and delete helper."
   (interactive)
@@ -230,10 +264,11 @@ Also nil `mastodon-auth--token-alist'."
     (define-key map (kbd "l")      #'recenter-top-bottom)
     ;; navigation between timelines
     (define-key map (kbd "#")      #'mastodon-tl-get-tag-timeline)
-    (define-key map (kbd "\"")     #'mastodon-tl-list-followed-tags)
-    (define-key map (kbd "C-\"")     #'mastodon-tl-jump-to-followed-tag)
+    (define-key map (kbd "C-#")    #'mastodon-tl-list-followed-tags)
+    ;; (define-key map (kbd "\"")     #'mastodon-tl-list-followed-tags)
+    (define-key map (kbd "C-\"")   #'mastodon-tl-jump-to-followed-tag)
     (define-key map (kbd "'")      #'mastodon-tl-followed-tags-timeline)
-    (define-key map (kbd "C-'")   #'mastodon-tl-tag-group-timeline)
+    (define-key map (kbd "C-'")    #'mastodon-tl-tag-group-timeline)
     (define-key map (kbd "A")      #'mastodon-profile-get-toot-author)
     (define-key map (kbd "F")      #'mastodon-tl-get-federated-timeline)
     (define-key map (kbd "H")      #'mastodon-tl-get-home-timeline)
@@ -255,6 +290,7 @@ Also nil `mastodon-auth--token-alist'."
     (define-key map (kbd "f")      #'mastodon-toot-toggle-favourite)
     (define-key map (kbd "k")      #'mastodon-toot-toggle-bookmark)
     (define-key map (kbd "r")      #'mastodon-toot-reply)
+    (define-key map (kbd "\"")     #'mastodon-toot-quote)
     (define-key map (kbd "C")      #'mastodon-toot-copy-toot-url)
     (define-key map (kbd "o")      #'mastodon-toot-browse-toot-url)
     (define-key map (kbd "v")      #'mastodon-tl-poll-vote)
@@ -415,13 +451,15 @@ FORCE means to fetch from the server in any case and update
   (alist-get 'version (mastodon-instance-data)))
 
 ;;;###autoload
-(defun mastodon-toot (&optional user reply-to-id reply-json)
+(defun mastodon-toot (&optional user reply-to-id reply-json
+                                quote-id quote-json visibility)
   "Update instance with new toot. Content is captured in a new buffer.
 If USER is non-nil, insert after @ symbol to begin new toot.
 If REPLY-TO-ID is non-nil, attach new toot to a conversation.
 If REPLY-JSON is the json of the toot being replied to."
   (interactive)
-  (mastodon-toot--compose-buffer user reply-to-id reply-json))
+  (mastodon-toot--compose-buffer user reply-to-id reply-json
+                                 nil nil quote-id quote-json visibility))
 
 ;;;###autoload
 (defun mastodon-notifications-get (&optional type buffer-name max-id)
@@ -451,9 +489,18 @@ MAX-ID is a request parameter for pagination."
          "v1"
        "v2"))
     (with-current-buffer (get-buffer-create buffer)
-      (use-local-map mastodon-notifications--map))
+      (use-local-map mastodon-notifications--map)
+      (when mastodon-notifications-notify-shown
+        (setq mastodon-notifications-notify-shown nil)))
     (message "Loading your notifications... Done")))
 
+;;;###autoload
+(defun mastodon-notifications-check ()
+  "Check for the number of unread notifications."
+  (interactive)
+  (let ((count (mastodon-notifications--get-unread-count)))
+    (message "Unread notifications: %s" count)))
+
 ;; URL lookup: should be available even if `mastodon.el' not loaded:
 
 ;;;###autoload
@@ -574,7 +621,10 @@ Calls `mastodon-tl--get-buffer-type', which see."
   ;; make `thing-at-point' functions work:
   (setq-local thing-at-point-provider-alist
               (append thing-at-point-provider-alist
-                      '((url . mastodon--url-at-point)))))
+                      '((url . mastodon--url-at-point))))
+  ;; notifs timer
+  (when mastodon-notifications-alerts
+    (mastodon-notifications--update-with-timer)))
 
 ;;;###autoload
 (add-hook 'mastodon-mode-hook #'mastodon-mode-hook-fun)
